/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;

import com.patrikdufresne.managers.ArchivableObject;

/**
 * This class represent a position that may be filled by an employee. The
 * position is mostly use to keep track of the employee's qualification.
 */
@Entity
public class Position extends ArchivableObject {

    /**
     * Classified property
     */
    public static final String CLASSIFIED = "classified"; //$NON-NLS-1$

    /**
     * Name property
     */
    public static final String NAME = "name"; //$NON-NLS-1$
    /**
     * RefId property
     */
    public static final String REFID = "refId"; //$NON-NLS-1$
    /**
     * Section property
     */
    public static final String SECTION = "section"; //$NON-NLS-1$

    /**
     * Swappable property.
     */
    public static final String SWAPPABLE = "swappable"; //$NON-NLS-1$

    private static final long serialVersionUID = 6472370093611592367L;

    private boolean classified;

    /**
     * Define a display name for the position.
     */
    private String name;

    /**
     * Define a reference number to the position.
     */
    private String refId;

    /**
     * Define the position's section.
     */
    private Section section;

    /**
     * Define if the position is swappable. A swappable position allow
     * associated tasks to be part of multiple shift.
     */
    private boolean swappable;

    /**
     * Getter for classified position.
     * 
     * @return True if the position is classified.
     */
    public boolean getClassified() {
        return this.classified;
    }

    /**
     * Getter of the property <tt>name</tt>
     * 
     * @return Returns the name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Getter of the property <tt>refId</tt>
     * 
     * @return Returns the refId.
     */
    @Column(unique = true)
    public String getRefId() {
        return this.refId;
    }

    /**
     * Getter of the property <tt>section</tt>
     * 
     * @return Returns the section.
     */
    @ManyToOne(optional = false)
    public Section getSection() {
        return this.section;
    }

    /**
     * Check if the position is swappable.
     * 
     * @return
     */
    public boolean getSwappable() {
        return this.swappable;
    }

    /**
     * Setter for classified position.
     * 
     * @param classified
     *            True to make the position classified
     */
    public void setClassified(boolean classified) {
        this.changeSupport.firePropertyChange(CLASSIFIED, this.classified, this.classified = classified);
    }

    /**
     * Setter of the property <tt>name</tt>
     * 
     * @param name
     *            The name to set.
     */
    public void setName(String name) {
        this.changeSupport.firePropertyChange(NAME, this.name, this.name = name);
    }

    /**
     * Setter of the property <tt>refId</tt>
     * 
     * @param refId
     *            The refId to set.
     */
    public void setRefId(String refId) {
        this.changeSupport.firePropertyChange(REFID, this.refId, this.refId = refId);
    }

    /**
     * Setter of the property <tt>section</tt>
     * 
     * @param section
     *            The section to set.
     */
    public void setSection(Section section) {
        this.changeSupport.firePropertyChange(SECTION, this.section, this.section = section);
    }

    /**
     * Sets the swappable flag.
     * 
     * @param swappable
     */
    public void setSwappable(boolean swappable) {
        this.changeSupport.firePropertyChange(SWAPPABLE, this.swappable, this.swappable = swappable);
    }

    @Override
    public String toString() {
        return "Position [id=" + this.id + ", refId=" + this.refId + ", name=" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                + this.name
                + "]"; //$NON-NLS-1$
    }

}
