/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.planimod.core;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import com.patrikdufresne.managers.AbstractManager;
import com.patrikdufresne.managers.ManagerContext;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.Query;
import com.patrikdufresne.util.BidiMultiHashMap;
import com.patrikdufresne.util.BidiMultiMap;

/**
 * Object manager for product.
 * 
 * @author Patrik Dufresne
 */
public class ProductManager extends AbstractManager<Product> {

    public ProductManager(PlanimodManagers managers) {
        super(managers);
    }

    /**
     * This function is used to copy a given product.
     * 
     * @param product
     *            The original product
     * @throws ManagerException
     */
    public Product copy(final Product product) throws ManagerException {
        return getManagers().query(new Query<Product>() {
            @Override
            public Product run() throws ManagerException {
                Product newProduct = new Product();
                newProduct.setFamily(product.getFamily());
                newProduct.setName(product.getName());
                add(Arrays.asList(newProduct));

                /*
                 * Copy the productPositions
                 */
                List<ProductPosition> list = getManagers().getProductPositionManager().listByProduct(product);
                for (ProductPosition pp : list) {
                    // Proceed with a copy
                    ProductPosition newPp = new ProductPosition();
                    newPp.setProduct(newProduct);
                    newPp.setPosition(pp.getPosition());
                    newPp.setNumber(pp.getNumber());
                    getManagers().getProductPositionManager().add(Arrays.asList(newPp));
                }
                return newProduct;
            }
        });
    }

    /**
     * Return the managers
     * 
     * @return
     */
    @Override
    public PlanimodManagers getManagers() {
        return (PlanimodManagers) super.getManagers();
    }

    /**
     * Return a table of shift and Product entry
     * 
     * @param shifts
     * @throws ManagerException
     */
    public Collection<ProductEntry> listProductTable(final Collection<Product> products) throws ManagerException {

        return getManagers().query(new Query<List<ProductEntry>>() {

            @Override
            public List<ProductEntry> run() throws ManagerException {

                // Query database
                org.hibernate.Query query = ManagerContext.getDefaultSession().getNamedQuery("listProductTable"); //$NON-NLS-1$
                query.setParameterList("products", products); //$NON-NLS-1$

                // Hash this data using a bidi-multi-map
                List<ProductEntry> table = new ArrayList<ProductEntry>();
                for (Object[] row : (List<Object[]>) query.list()) {
                    Product product = (Product) row[0];
                    Integer count = Integer.valueOf(((Long) row[1]).intValue());
                    table.add(new ProductEntry(product, count));
                }
                return table;
            }

        });

    }

    /**
     * Return a table of production event.
     * 
     * @param events
     * @return
     * @throws ManagerException
     */
    public Map<ProductionEvent, ProductEntry> listProductTableByProductionEvent(Collection<ProductionEvent> events) throws ManagerException {
        // Collect product.
        Set<Product> products = new HashSet<Product>();
        for (ProductionEvent e : events) {
            products.add(e.getProduct());
        }
        // Query table
        Collection<ProductEntry> entries = listProductTable(products);
        // Build the map
        Map<ProductionEvent, ProductEntry> table = new HashMap<ProductionEvent, ProductEntry>(events.size());
        for (ProductionEvent e : events) {
            for (ProductEntry entry : entries) {
                if (entry.getKey().equals(e.getProduct())) {
                    table.put(e, entry);
                    break;
                }
            }
        }
        return table;
    }

    /**
     * Return a table of shift and Product entry
     * 
     * @param shifts
     * @throws ManagerException
     */
    public BidiMultiMap<Shift, Entry<Product, Integer>> listProductTableByShifts(final Collection<Shift> shifts) throws ManagerException {

        return getManagers().query(new Query<BidiMultiMap<Shift, Entry<Product, Integer>>>() {

            @Override
            public BidiMultiMap<Shift, Entry<Product, Integer>> run() throws ManagerException {

                // Query database
                org.hibernate.Query query = ManagerContext.getDefaultSession().getNamedQuery("listProductTableByShifts"); //$NON-NLS-1$
                query.setParameterList("shifts", shifts); //$NON-NLS-1$

                // Hash this data using a bidi-multi-map
                BidiMultiMap<Shift, Entry<Product, Integer>> table = new BidiMultiHashMap<Shift, Entry<Product, Integer>>();
                for (Object[] row : (List<Object[]>) query.list()) {
                    Shift shift = (Shift) row[0];
                    Product product = (Product) row[1];
                    Integer count = Integer.valueOf(((Long) row[2]).intValue());
                    table.put(shift, new ProductEntry(product, count));
                }
                return table;
            }

        });

    }

    @Override
    public Class<Product> objectClass() {
        return Product.class;
    }

    /**
     * Override default implementation to delete the product position too.
     */
    @Override
    protected void preRemoveObject(Product t) throws ManagerException {
        ManagerContext.getDefaultSession().refresh(t);
        // This implementation will remove associate ProductPosition object
        Collection<ProductPosition> pps = getManagers().getProductPositionManager().listByProduct(t);
        getManagers().getProductPositionManager().remove(pps);
    }
}
