/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.patrikdufresne.ilp.Linear;

/**
 * This runnable is used to check if every tasks can be assigned to an employee.
 * 
 * @author Patrik Dufresne
 */
@SuppressWarnings("unqualified-field-access")
public class CheckTaskAssigned extends AbstractConstraintRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(CheckTaskAssigned.class);

    /**
     * The constraint's name.
     */
    private static final String CONSTRAINT_NAME = "CheckTaskAssigned: SUM from{i=0} to{n} SUM from{k=0} to{p} xijk >= %d"; //$NON-NLS-1$

    /**
     * Default constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public CheckTaskAssigned(ProblemBuilder problem) {
        super(problem);
        setFeasibilityPumpHeuristic(true);
    }

    /**
     * This implementation does nothing.
     */
    @Override
    protected void feasible() {
        // Every task may be assigned. There is no need to remove the
        // constraint.
        LOGGER.info("All task can be assigned."); //$NON-NLS-1$
    }

    @Override
    protected boolean setUp() {
        // Create a linear for the sum of every tasks
        Linear linear = pb.lp.createLinear();
        for (int i = 0; i < pb.xijk.length; i++) {
            for (int j = 0; j < pb.xijk[i].length; j++) {
                for (int k = 0; k < pb.xijk[i][j].length; k++) {
                    if (pb.xijk[i][j][k] != null) {
                        linear.add(pb.lp.createTerm(ONE, pb.xijk[i][j][k]));
                    }
                }
            }
        }
        if (linear.size() == 0) {
            feasible = true;
            return false;
        }
        // Create the following constraint
        // SUM xijk >= nbTask
        Number number = Integer.valueOf(pb.tasks.size());
        String name = String.format(CONSTRAINT_NAME, number);
        this.constraint = pb.lp.addConstraint(name, linear, number, null);
        // Reset the objective function
        pb.lp.setObjectiveLinear(null);
        LOGGER.debug("Check if every task can be assigned."); //$NON-NLS-1$
        return true;
    }

    /**
     * If it's impossible to assign all tasks, release the constraint.
     * 
     * @throws GeneratePlanifException
     */
    @Override
    protected void unfeasible() throws GeneratePlanifException {
        super.unfeasible();
        LOGGER.info("Can't assign all task."); //$NON-NLS-1$
    }
}