/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This runnable is used to assign the employee according to it's preferences.
 * 
 * @author Patrik Dufresne
 */
public class MaxEmployeePreferences extends SolveRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(MaxEmployeePreferences.class);

    /**
     * Public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public MaxEmployeePreferences(ProblemBuilder problem) {
        super(problem);
    }

    /**
     * This function is called to apply the non-availabilities for the given employee.
     * 
     * @throws GeneratePlanifException
     */
    private void handleNonAvailabilities(int i) throws GeneratePlanifException {
        // Check if the employee is available
        if (!(pb.fixNonAvailabilitiesVariables(i))) {
            return;
        }
        // Clear the employee from current to less senior. We do not need to
        // recompute the current employee.
        pb.maxSeniorEmployee.release(i + 1);
        pb.minSwappable.release();
        pb.minEmployee.release();
        pb.maxTaskAssigned.release();
        pb.maxTaskAssigned.run();
        pb.minEmployee.run();
        pb.minSwappable.run();
        pb.maxSeniorEmployee.run(i + 1);
    }

    @Override
    protected boolean runSolver() throws GeneratePlanifException {
        // Loop on employee - the list is sorted by seniority
        for (int i = 0; i < pb.xi.length; i++) {
            // Update monitor
            pb.worked(ProblemBuilder.UNIT);
            // Maximize the preferred position
            MaxPreferredPosition solver = new MaxPreferredPosition(pb, i);
            solver.run();
            if (solver.isFeasible()) {
                // Apply the non-availability.
                handleNonAvailabilities(i);
                // A solution as been found with the employee working on
                // it's preferred position. So there is not need to optimize
                // the preferred team.
                continue;
            }
            // Maximize the preferred team
            MaxPreferredTeams maxPreferredTeam = new MaxPreferredTeams(pb, i);
            maxPreferredTeam.run();
            // Apply the non-availability.
            handleNonAvailabilities(i);
        }
        // Loop on employee
        for (int i = 0; i < pb.xi.length; i++) {
            pb.worked(ProblemBuilder.UNIT);
            // Maximize the employee preferred section.
            MaxPreferredSection maxPreferredSection = new MaxPreferredSection(pb, i);
            maxPreferredSection.run();
        }
        return true;
    }

    @Override
    protected boolean setUp() throws GeneratePlanifException {
        LOGGER.debug("Maximize employee preferences."); //$NON-NLS-1$
        return true;
    }
}