/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.planimod.ui;

import static com.patrikdufresne.util.Localized._;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Locale;

import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.util.ILogger;
import org.eclipse.jface.util.Policy;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.window.WindowManager;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.patrikdufresne.jface.dialogs.DetailMessageDialog;
import com.patrikdufresne.jface.dialogs.StatusHandlerDialog;
import com.patrikdufresne.managers.H2DBConfigurations;
import com.planimod.core.PlanimodManagers;
import com.planimod.ui.preference.PlanimodPreferences;
import com.planimod.ui.theme.Resources;

public class Main {

    /**
     * Define the application data directory name.
     */
    private static final String APP_USER_DATA = "planimod"; //$NON-NLS-1$

    private static Logger logger = LoggerFactory.getLogger(Main.class);

    /**
     * Preference store of the application.
     */
    private static PreferenceStore pref;

    /**
     * Define the preference file's name.
     */
    private static final String PREF_FILENAME = "preferences"; //$NON-NLS-1$

    private static void handleOpenDatabaseException(Exception exception) {
        // Display a generic message for other error.
        DetailMessageDialog.openDetailWarning(null, _(Main.class, "Main.openDatabase.title"), _(Main.class, "Main.openDatabase.message"), //$NON-NLS-1$
                _(Main.class, "Main.openDatabase.connectionError"), //$NON-NLS-1$
                exception);
    }

    /**
     * Display a human readable error message.
     * 
     * @param exception
     * @return True if the database should be created.
     */
    private static boolean handleOpenDatabaseWithCreateTrialException(Exception exception) {
        // Check if the exception is causes by non-existing database. The only way I found is to parse the exception
        // message.
        Throwable cause = exception;
        while (cause != null) {
            if (cause.getMessage().contains("not found")) {
                DetailMessageDialog dlg = DetailMessageDialog.openDetailYesNoQuestion(null, _(Main.class, "Main.openDatabase.title"), _(
                        Main.class,
                        "Main.openDatabase.message"), _(Main.class, "Main.openDatabase.databaseNotFound"), null);
                return IDialogConstants.YES_ID == dlg.getReturnCode();
            }
            cause = cause.getCause();
        }

        handleOpenDatabaseException(exception);

        return false;
    }

    /**
     * Open the preference files.
     */
    public static void initPreference() {
        String osName = System.getProperty("os.name"); //$NON-NLS-1$
        String homeDir = System.getProperty("user.home"); //$NON-NLS-1$
        File prefFile;
        if (osName.equals("Linux")) { //$NON-NLS-1$
            File configDir = new File(homeDir, ".config/" + APP_USER_DATA); //$NON-NLS-1$
            configDir.mkdirs();
            prefFile = new File(configDir, PREF_FILENAME);
        } else {
            File configDir = new File(homeDir, APP_USER_DATA);
            configDir.mkdirs();
            prefFile = new File(configDir, PREF_FILENAME);
        }

        pref = new PreferenceStore(prefFile.getPath());
        try {
            pref.load();
        } catch (FileNotFoundException e) {
            // Swallow
        } catch (IOException e) {
            // Display error
            logger.error("Can't load the preferences.", e); //$NON-NLS-1$
        }

        PlanimodPreferences.setPreferenceStore(pref);
    }

    /**
     * @param args
     */
    public static void main(String[] args) {

        logger.info("Planimod is starting..."); //$NON-NLS-1$

        // Set the default locale to french
        Locale.setDefault(Locale.CANADA_FRENCH);

        // Init preferences
        initPreference();

        Display.setAppName(_(Main.class, "applicationName")); //$NON-NLS-1$

        final Display display = new Display();
        Realm.runWithDefault(SWTObservables.getRealm(Display.getCurrent()), new Runnable() {
            @Override
            public void run() {

                updateJFacePolicy();

                // Set default application image to be use by all JFace
                // Window
                Window.setDefaultImages(new Image[] {
                        Resources.getImage(Resources.APP_ICON_16),
                        Resources.getImage(Resources.APP_ICON_22),
                        Resources.getImage(Resources.APP_ICON_48),
                        Resources.getImage(Resources.APP_ICON_128) });

                WindowManager winManager = new WindowManager();
                PlanimodManagers managers = null;
                try {
                    if ((managers = openDatabase()) == null) {
                        // Exit application
                        savePreference();
                        return;
                    }

                    // Open Main Window
                    PlanimodMainWindow win = new PlanimodMainWindow(null);
                    win.setManagers(managers);
                    win.setWindowManager(winManager);
                    win.open();

                    // Event loop
                    while (winManager.getWindowCount() > 0) {
                        try {
                            if (!display.readAndDispatch()) display.sleep();
                        } catch (RuntimeException e) {
                            PlanimodPolicy.showException(e);
                        }
                    }

                    savePreference();

                } catch (Throwable e) {
                    logger.error("Fatal error.", e); //$NON-NLS-1$
                    DetailMessageDialog.openDetailWarning(null, _(Main.class, "fatalError.title"), _(Main.class, "fatalError.message"), null, e);
                } finally {
                    if (managers != null) {
                        managers.dispose();
                    }
                    winManager.close();
                    display.dispose();
                }
            }
        });
    }

    /**
     * Open a database. Returns True if a database got open. False if the operation have been cancel.
     */
    static PlanimodManagers openDatabase() {

        // Open DB selector
        PlanimodManagers managers = null;
        DatabaseSelectorDialog dlg = new DatabaseSelectorDialog(null);
        do {
            if (IDialogConstants.OK_ID != dlg.open() || dlg.getManagersConfig() == null) {
                // Operation cancel by user
                return null;
            }
            String url = dlg.getManagersConfig();
            try {
                // Open the database
                logger.info("Opening database: {}", url); //$NON-NLS-1$
                managers = new PlanimodManagers(H2DBConfigurations.create(url, false, true));
            } catch (Exception e) {
                logger.error("Error opening the database connection", e); //$NON-NLS-1$
                if (handleOpenDatabaseWithCreateTrialException(e)) {
                    try {
                        managers = new PlanimodManagers(H2DBConfigurations.create(url, true, true));
                    } catch (Exception e1) {
                        handleOpenDatabaseException(e1);
                    }
                }
            }
        } while (managers == null);

        return managers;
    }

    /**
     * Save the preferences.
     */
    public static void savePreference() {
        if (pref != null) {
            try {
                pref.save();
            } catch (IOException e) {
                // Print the error into logs
                logger.error("Can't save preferences.", e); //$NON-NLS-1$
            }
        }
    }

    /**
     * This function sets the status handler.
     */
    protected static void updateJFacePolicy() {
        /**
         * Redirect all the Jface logger into SLF4J logger.
         */
        Policy.setLog(new ILogger() {

            /**
             * Create a default logger for ILogger.
             */
            final transient Logger logger = LoggerFactory.getLogger(getClass());

            /**
             * This implementation is logging the status object using the logger.
             */
            @Override
            public void log(IStatus status) {
                switch (status.getSeverity()) {
                case IStatus.OK:
                case IStatus.INFO:
                    if (status.getException() == null) {
                        logger.info(status.getMessage());
                    } else {
                        logger.info(status.getMessage(), status.getException());
                    }
                    break;
                case IStatus.WARNING:
                    if (status.getException() == null) {
                        logger.warn(status.getMessage());
                    } else {
                        logger.warn(status.getMessage(), status.getException());
                    }
                    break;
                case IStatus.ERROR:
                    if (status.getException() == null) {
                        logger.error(status.getMessage());
                    } else {
                        logger.error(status.getMessage(), status.getException());
                    }
                    break;
                }
            }
        });
        /**
         * Redirect all the JFace exception to our status dialog.
         */
        Policy.setStatusHandler(StatusHandlerDialog.getStatusHandler());
    }
}
