/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.databinding;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.databinding.observable.Diffs;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.core.databinding.observable.set.ISetChangeListener;
import org.eclipse.core.databinding.observable.set.SetChangeEvent;
import org.eclipse.core.databinding.property.INativePropertyListener;
import org.eclipse.core.databinding.property.ISimplePropertyListener;
import org.eclipse.core.databinding.property.NativePropertyListener;
import org.eclipse.core.databinding.property.value.SimpleValueProperty;

/**
 * Property used to check if a set contains a given value.
 * 
 * @author Patrik Dufresne
 * 
 */
public class SetContainsPropertyValue extends SimpleValueProperty {

    private class Listener extends NativePropertyListener implements ISetChangeListener {

        List<Object> knownValue;

        public Listener(ISimplePropertyListener listener) {
            super(SetContainsPropertyValue.this, listener);
        }

        @Override
        protected void doAddTo(Object source) {
            if (this.knownValue == null) {
                // Add listener to the set
                this.knownValue = new ArrayList<Object>();
                SetContainsPropertyValue.this.set.addSetChangeListener(this);
            }
            this.knownValue.add(source);
        }

        @Override
        protected void doRemoveFrom(Object source) {
            if (this.knownValue != null) {
                this.knownValue.remove(source);
                if (this.knownValue.size() == 0) {
                    this.knownValue = null;
                    SetContainsPropertyValue.this.set.removeSetChangeListener(this);
                }
            }
        }

        @Override
        public void handleSetChange(SetChangeEvent event) {
            for (Object val : event.diff.getAdditions()) {
                if (this.knownValue.contains(val)) {
                    fireChange(val, Diffs.createValueDiff(Boolean.FALSE, Boolean.TRUE));
                }
            }
            for (Object val : event.diff.getRemovals()) {
                if (this.knownValue.contains(val)) {
                    fireChange(val, Diffs.createValueDiff(Boolean.TRUE, Boolean.FALSE));
                }
            }
        }

    }

    IObservableSet set;

    public SetContainsPropertyValue(IObservableSet set) {
        this.set = set;
    }

    @Override
    public Object getValueType() {
        return Boolean.TYPE;
    }

    @Override
    protected Object doGetValue(Object source) {
        return Boolean.valueOf(this.set.contains(source));
    }

    @Override
    protected void doSetValue(Object source, Object value) {
        if (Boolean.TRUE.equals(value)) {
            this.set.add(source);
        } else if (Boolean.FALSE.equals(value)) {
            this.set.remove(source);
        }
    }

    @Override
    public INativePropertyListener adaptListener(ISimplePropertyListener listener) {
        return new Listener(listener);
    }

}
