/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package com.planimod.ui.views.shifts;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.window.IShellProvider;
import org.eclipse.jface.window.Window;

import com.patrikdufresne.managers.ManagedObject;
import com.patrikdufresne.managers.jface.AbstractAddAction;
import com.patrikdufresne.util.Localized;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Shift;
import com.planimod.core.Team;
import com.planimod.core.TimeRanges;
import com.planimod.ui.theme.Resources;

/**
 * Action to create a new non-availability event and add it to the database.
 * 
 * @author Patrik Dufresne
 * 
 */
public class AddShiftAction extends AbstractAddAction {
    /**
     * Property name of an action's date (value <code>"date"</code>). The date property is used sets a default time
     * range.
     */
    public static final String DATE = "date"; //$NON-NLS-1$
    /**
     * A date used as a hint to create the new shift event.
     */
    private Date date;
    /**
     * Temporary end date.
     */
    private Date end;

    /**
     * Temporary shift object, or null.
     */
    private Team shift;

    /**
     * Temporary start date.
     */
    private Date start;

    /**
     * Create a new action.
     * 
     * @param shellProvider
     *            a shell provider in case the action need to display a message box.
     */
    public AddShiftAction(PlanimodManagers managers, IShellProvider shellProvider, ISelectionProvider selectionProvider) {
        super(managers, shellProvider, selectionProvider);
        setText(Localized.get(AddShiftAction.class, "AddShiftAction.text")); //$NON-NLS-1$
        setToolTipText(Localized.get(AddShiftAction.class, "AddShiftAction.tooltipText")); //$NON-NLS-1$
        setImageDescriptor(Resources.getImageDescriptor(Resources.ICON_LIST_ADD_16));
    }

    /**
     * This implementation always return true.
     * 
     * @see net.ekwos.gymkhana.ui.views.AbstractAddAction#canCreateObject()
     */
    @Override
    protected boolean canCreateObject() {
        return true;
    }

    /**
     * This implementation create non-availability event object.
     * 
     * @see net.ekwos.gymkhana.ui.views.AbstractAddAction#createObjects()
     */
    @Override
    protected List<? extends ManagedObject> createObjects() {
        List<ManagedObject> list = new ArrayList<ManagedObject>();
        // Create the event
        Shift event = new Shift();
        event.setTeam(this.shift);
        event.setStartDate(this.start);
        event.setEndDate(this.end);
        list.add(event);
        return list;
    }

    /**
     * Returns the current date state of null if not define.
     * 
     * @return the date state or null.
     */
    public Date getDate() {
        return this.date;
    }

    @Override
    public void run() {
        ShiftDialog dlg = new ShiftDialog(getShellProvider(), (PlanimodManagers) getManagers(), ShiftDialog.NEW);
        Date curDate = this.date == null ? new Date() : this.date;
        dlg.setStart(curDate);
        dlg.setEnd(curDate);
        dlg.setShift(null);
        if (dlg.open() != Window.OK) {
            return;
        }

        this.start = TimeRanges.removeSecond(dlg.getStart());
        this.end = TimeRanges.removeSecond(dlg.getEnd());
        this.shift = dlg.getShift();

        if (this.shift == null || this.start == null || this.end == null || this.start.compareTo(this.end) > 0) {
            return;
        }

        super.run();
    }

    /**
     * Sets a default date value.
     * 
     * @param date
     *            the date value or null
     */
    public void setDate(Date date) {
        if ((this.date == null && date != null) || (this.date != null && date == null) || (this.date != null && date != null && !date.equals(this.date))) {
            Date oldValue = this.date;
            this.date = date;
            firePropertyChange(DATE, oldValue, this.date);
        }
    }
}